import Debug from 'debug';
import _ from 'lodash';
import path from 'path';
import ms from 'ms';
import Promise from 'bluebird';
import { EventEmitter } from 'fbemitter'
import foreach from 'generator-foreach';
import _termkit, { ScreenBuffer } from 'terminal-kit';
import co from 'co';
import _say from 'say';
import map from 'co-map';
const imageToAscii = Promise.promisify(require('image-to-ascii'));
const fs = Promise.promisifyAll(require('fs'));
const say = Promise.promisifyAll(_say);
const termkit = Promise.promisifyAll(_termkit);
const debug = Debug('alphabet-cli:App')
const VOICE = 'kal_diphone';

export default class App extends EventEmitter {
  constructor(){
    super();
    let ctx = this;
    return co(function*(){
      yield ctx.init();
    })
    .catch((err) => {
      console.err(err)
      process.exit()
    })
  }
  *init(){
    this._validCount = 0;
    this._count = 0;
    this.addListener('nextcard', this.nextCard.bind(this));
    this.sprites = {};
    this.term = yield termkit.getDetectedTerminalAsync();
    let {
      term
      , inputs
    } = this;
    this.viewport = ScreenBuffer.create( {
			dst: term ,
			width: Math.min( term.width ) ,
			height: Math.min( term.height - 1 ) ,
			y: 2
		} ) ;
    term.hideCursor();
		term.grabInput();
		term.on( 'key' , inputs.bind(this) ) ;
    yield this.initAssets();
    this.nextCard();
    this.initViewPort();
    this.initBackground();
    this.animate();
    debug('initialized...')
  }
  *initAssets(){
    let {
      term
    } = this;
    let assetDir = `${__dirname}/buffers`;
    debug(assetDir)
    let assets = yield fs.readdirAsync(assetDir);
    let ascii = {};
    assets = assets.map((file) => {
      return path.resolve(`${__dirname}/buffers/${file}`)
    })
    debug(assets)
    assets.forEach(function(bufferFile){
      var buffer = fs.readFileSync(bufferFile, 'utf8')
      bufferFile = bufferFile.match(/([A-Z])\.txt$/)[1];
      bufferFile = bufferFile.toLowerCase();
      ascii[bufferFile] = buffer;
    })
    this._assets = ascii;
  }
  nextCard(){
    debug('#nextCard')
    this._count ++;
    let cardKeys = _.keys(this._assets);
    let keyNum = _.random(0, cardKeys.length-1)
    this._cardKey = cardKeys[keyNum];
    let card = this._assets[this._cardKey];
    this._nextCard = card;
    this.sprites.currentCard = card
    say.speak(VOICE, this._cardKey);
  }
  initViewPort(){
    let {
      term
    } = this;
    this.viewport = ScreenBuffer.create( {
			dst: term ,
			width: Math.min( term.width ) ,
			height: Math.min( term.height - 1 ) ,
			y: 2
		} ) ;
  }
  initBackground(){
    let {
      viewport
    } = this;
    this.sprites.background = ScreenBuffer.create( {
      width: viewport.width
      , height: viewport.height
    } ) ;
  }
  draw(){
    let {
      sprites: {
        background
        , currentCard
       }
      , viewport
      , term
      , _count: count
      , _validCount: validCount
      , _percent: percent
    } = this;
    // debug(this.sprites.background)
    background.draw( { dst: viewport , tile: true } ) ;
    // currentCard.draw({
    //   dst: viewport
    //   , blending: true
    // });
    console.log(this._nextCard)
    viewport.draw( { delta: true } ) ;
    debug('#draw')
    // debug(background)
    term.moveTo.eraseLine.bgWhite.green( 1 , 1 , `results: ${validCount} of ${count}, ${percent}%`)
  }
  animate(){
    debug('#animate')
  	this.draw();
  	// nextPosition();
  	setTimeout( this.animate.bind(this) , 50 ) ;
  }
  checkCard(value){
    let {
      _cardKey: key
    } = this;
    if(value === key)
    this.valid();
    else
    this.invalid();
  }
  calcPercent(){
    this._percent = Math.round(this._validCount/this._count*100)
  }
  valid(){
    var ctx = this;
    this._validCount ++;
    this.calcPercent();
    return co(function* (){
      yield say.speakAsync(VOICE, 'Good job!');
      ctx.emit('nextcard');
    });
  }
  invalid(){
    var ctx = this;
    this._count ++;
    this.calcPercent();
    return co(function* (){
      yield say.speakAsync(VOICE, 'Try again!')
    })
  }
  inputs(key){
    let {
      sprites
    } = this;
    if(key.match(/[a-z]/))
    return this.checkCard(key);
    switch ( key )
  	{
  		case 'CTRL_C':
  			this.terminate() ;
  			break ;
  	}
  }
  terminate(){
    let {
      term
    } = this;
  	//term.fullscreen( false ) ;
  	term.hideCursor( false ) ;
  	term.grabInput( false ) ;
    co(function*(){
      // yield say.speakAsync('Alex','hola Santy');
    	setTimeout( function() {
    		term.moveTo( 1 , term.height , '\n\n' ) ;
    		process.exit() ;
    	} , 100 ) ;
    })
  }
}
